@@ -14,8 +14,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import os
 import json
+import os
 import shutil
 import uuid
 import warnings
@@ -23,14 +23,15 @@ from collections import OrderedDict
 
 import numpy as np
 import pandas as pd
+from mleap.bundle.serialize import MLeapSerializer, MLeapDeserializer, Vector
+from sklearn.impute import SimpleImputer
+from sklearn.preprocessing import StandardScaler, MinMaxScaler, Binarizer, PolynomialFeatures
 from sklearn.preprocessing.data import BaseEstimator, TransformerMixin
-from sklearn.preprocessing import StandardScaler, MinMaxScaler, Imputer, Binarizer, PolynomialFeatures
 from sklearn.preprocessing.data import OneHotEncoder
 from sklearn.preprocessing.label import LabelEncoder
-from mleap.bundle.serialize import MLeapSerializer, MLeapDeserializer, Vector
 from sklearn.utils import column_or_1d
-from sklearn.utils.validation import check_is_fitted
 from sklearn.utils.fixes import np_version
+from sklearn.utils.validation import check_is_fitted
 
 
 class ops(object):
@@ -117,10 +118,10 @@ setattr(MinMaxScaler, 'serialize_to_bundle', serialize_to_bundle)
 setattr(MinMaxScaler, 'deserialize_from_bundle', deserialize_from_bundle)
 setattr(MinMaxScaler, 'serializable', True)
 
-setattr(Imputer, 'op', ops.IMPUTER)
-setattr(Imputer, 'mlinit', mleap_init)
-setattr(Imputer, 'serialize_to_bundle', serialize_to_bundle)
-setattr(Imputer, 'serializable', True)
+setattr(SimpleImputer, 'op', ops.IMPUTER)
+setattr(SimpleImputer, 'mlinit', mleap_init)
+setattr(SimpleImputer, 'serialize_to_bundle', serialize_to_bundle)
+setattr(SimpleImputer, 'serializable', True)
 
 setattr(OneHotEncoder, 'op', ops.ONE_HOT_ENCODER)
 setattr(OneHotEncoder, 'mlinit', mleap_init)
@@ -531,15 +532,19 @@ class ImputerSerializer(MLeapSerializer):
         self.serializable = False
 
     def serialize_to_bundle(self, transformer, path, model_name):
+        if transformer.strategy == 'most_frequent' or transformer.strategy == 'constant':
+            raise NotImplementedError(f"Scikit-learn's Imputer strategy `{transformer.strategy}` is not supported by MLeap")
+        if transformer.add_indicator:
+            raise NotImplementedError("Scikit-learn's Imputer parameter `add_indicator` is not supported by MLeap")
+        if len(transformer.statistics_.tolist()) != 1:
+            raise NotImplementedError("MLeap's Imputer only supports imputing a single feature at a time")
 
-        # compile tuples of model attributes to serialize
         attributes = list()
         attributes.append(('strategy', transformer.strategy))
         attributes.append(('surrogate_value', transformer.statistics_.tolist()[0]))
-        if transformer.missing_values != "NaN":
+        if not np.isnan(transformer.missing_values):
             attributes.append(('missing_value', transformer.missing_values))
 
-        # define node inputs and outputs
         inputs = [{
                   "name": transformer.input_features,
                   "port": "input"
@@ -620,22 +625,32 @@ class OneHotEncoderSerializer(MLeapSerializer, MLeapDeserializer):
     A one-hot encoder maps a single column of categorical indices to a
     column of binary vectors, which can be re-assamble back to a DataFrame using a ToDense transformer.
     """
+
     def __init__(self):
         super(OneHotEncoderSerializer, self).__init__()
 
     def serialize_to_bundle(self, transformer, path, model_name):
+        if len(transformer.categories_) != 1:
+            raise NotImplementedError("MLeap can only one-hot encode a single feature at a time")
+        single_feature_categories = transformer.categories_[0]
+        if not np.array_equal(single_feature_categories, np.arange(single_feature_categories.size)):
+            raise ValueError(f"Categories {single_feature_categories} do not form a valid index range")
+        if transformer.drop is not None:
+            raise NotImplementedError("Scikit-learn's OneHotEncoder `drop` parameter is not supported by MLeap")
+        if transformer.dtype != np.float64:
+            raise NotImplementedError("Scikit-learn's OneHotEncoder `dtype` parameter is not supported by MLeap")
 
-        # compile tuples of model attributes to serialize
         attributes = list()
-        attributes.append(('size', transformer.n_values_.tolist()[0]))
-        # setting drop_last to True, so to that 1HE maintains parity with MLeap/Spark
-        attributes.append(('drop_last', True))
+        attributes.append(('size', single_feature_categories.size))
         if transformer.handle_unknown == 'ignore':
+            # MLeap's OneHotEncoderModel adds an extra column when keeping invalid data
+            # so we drop that extra column to match sklearn's ignore behavior
             attributes.append(('handle_invalid', 'keep'))
+            attributes.append(('drop_last', True))
         else:
             attributes.append(('handle_invalid', 'error'))
+            attributes.append(('drop_last', False))
 
-        # define node inputs and outputs
         inputs = [{
                   "name": transformer.input_features,
                   "port": "input"
@@ -651,20 +666,19 @@ class OneHotEncoderSerializer(MLeapSerializer, MLeapDeserializer):
     def deserialize_from_bundle(self, transformer, node_path, node_name):
 
         attributes_map = {
-            'size': 'n_values_'
+            'size': 'categories_',
+            'handle_invalid': 'handle_unknown',
         }
 
         full_node_path = os.path.join(node_path, node_name)
         transformer = self.deserialize_single_input_output(transformer, full_node_path, attributes_map)
 
-        # Set Sparse = False
-        transformer.sparse = False
+        transformer.categories_ = np.asarray([range(transformer.categories_)])
+        if transformer.handle_unknown == 'keep':
+            transformer.handle_unknown = 'ignore'
+        transformer.drop_idx_ = None
 
-        # Set Feature Indices
-        n_values = np.hstack([[0], [transformer.n_values_]])
-        indices = np.cumsum(n_values)
-        transformer.feature_indices_ = indices
-        transformer.active_features_ = range(0, transformer.n_values_)
+        transformer.sparse = False
 
         return transformer
 
